@@ -23,7 +23,7 @@ options:
   description:
     description:
       - Specifies descriptive text that identifies the pool.
-    version_added: "2.3"
+    version_added: 2.3
   name:
     description:
       - Pool name
@@ -34,7 +34,7 @@ options:
     description:
       - Load balancing method. When creating a new pool, if this value is not
         specified, the default of C(round-robin) will be used.
-    version_added: "1.3"
+    version_added: 1.3
     choices:
       - dynamic-ratio-member
       - dynamic-ratio-node
@@ -54,7 +54,7 @@ options:
       - ratio-session
       - round-robin
       - weighted-least-connections-member
-      - weighted-least-connections-nod
+      - weighted-least-connections-node
   monitor_type:
     description:
       - Monitor rule type when C(monitors) is specified.
@@ -69,32 +69,32 @@ options:
         or already existing on the device.
       - Both C(single) and C(and_list) are functionally identical since BIG-IP
         considers all monitors as "a list".
-    version_added: "1.3"
+    version_added: 1.3
     choices: ['and_list', 'm_of_n', 'single']
   quorum:
     description:
       - Monitor quorum value when C(monitor_type) is C(m_of_n).
       - Quorum must be a value of 1 or greater when C(monitor_type) is C(m_of_n).
-    version_added: "1.3"
+    version_added: 1.3
   monitors:
     description:
       - Monitor template name list. If the partition is not provided as part of
         the monitor name, then the C(partition) option will be used instead.
-    version_added: "1.3"
+    version_added: 1.3
   slow_ramp_time:
     description:
       - Sets the ramp-up time (in seconds) to gradually ramp up the load on
         newly added or freshly detected up pool members.
-    version_added: "1.3"
+    version_added: 1.3
   reselect_tries:
     description:
       - Sets the number of times the system tries to contact a pool member
         after a passive failure.
-    version_added: "2.2"
+    version_added: 2.2
   service_down_action:
     description:
       - Sets the action to take when node goes down in pool.
-    version_added: "1.3"
+    version_added: 1.3
     choices:
       - none
       - reset
@@ -124,8 +124,26 @@ options:
         that are numbers.
       - Data will be persisted, not ephemeral.
     version_added: 2.5
+  priority_group_activation:
+    description:
+      - Specifies whether the system load balances traffic according to the priority
+        number assigned to the pool member.
+      - When creating a new pool, if this parameter is not specified, the default of
+        C(0) will be used.
+      - To disable this setting, provide the value C(0).
+      - Once you enable this setting, you can specify pool member priority when you
+        create a new pool or on a pool member's properties screen.
+      - The system treats same-priority pool members as a group.
+      - To enable priority group activation, provide a number from C(0) to C(65535)
+        that represents the minimum number of members that must be available in one
+        priority group before the system directs traffic to members in a lower
+        priority group.
+      - When a sufficient number of members become available in the higher priority
+        group, the system again directs traffic to the higher priority group.
+    aliases:
+      - minimum_active_members
+    version_added: 2.6
 notes:
-  - Requires BIG-IP software version >= 12.
   - To add members do a pool, use the C(bigip_pool_member) module. Previously, the
     C(bigip_pool) module allowed the management of users, but this has been removed
     in version 2.5 of Ansible.
@@ -144,7 +162,7 @@ EXAMPLES = r'''
     state: present
     name: my-pool
     partition: Common
-    lb_method: least-connection-member
+    lb_method: least-connections-member
     slow_ramp_time: 120
   delegate_to: localhost
 
@@ -307,6 +325,11 @@ metadata:
   returned: changed
   type: dict
   sample: {'key1': 'foo', 'key2': 'bar'}
+priority_group_activation:
+  description: The new minimum number of members to activate the priorty group.
+  returned: changed
+  type: int
+  sample: 10
 '''
 
 import re
@@ -315,42 +338,31 @@ from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.basic import env_fallback
 from ansible.module_utils.six import iteritems
 
-HAS_DEVEL_IMPORTS = False
-
 try:
-    # Sideband repository used for dev
     from library.module_utils.network.f5.bigip import HAS_F5SDK
     from library.module_utils.network.f5.bigip import F5Client
     from library.module_utils.network.f5.common import F5ModuleError
     from library.module_utils.network.f5.common import AnsibleF5Parameters
     from library.module_utils.network.f5.common import cleanup_tokens
-    from library.module_utils.network.f5.common import fqdn_name
+    from library.module_utils.network.f5.common import fq_name
     from library.module_utils.network.f5.common import f5_argument_spec
     try:
         from library.module_utils.network.f5.common import iControlUnexpectedHTTPError
     except ImportError:
         HAS_F5SDK = False
-    HAS_DEVEL_IMPORTS = True
 except ImportError:
-    # Upstream Ansible
     from ansible.module_utils.network.f5.bigip import HAS_F5SDK
     from ansible.module_utils.network.f5.bigip import F5Client
     from ansible.module_utils.network.f5.common import F5ModuleError
     from ansible.module_utils.network.f5.common import AnsibleF5Parameters
     from ansible.module_utils.network.f5.common import cleanup_tokens
-    from ansible.module_utils.network.f5.common import fqdn_name
+    from ansible.module_utils.network.f5.common import fq_name
     from ansible.module_utils.network.f5.common import f5_argument_spec
     try:
         from ansible.module_utils.network.f5.common import iControlUnexpectedHTTPError
     except ImportError:
         HAS_F5SDK = False
 
-try:
-    from netaddr import IPAddress, AddrFormatError
-    HAS_NETADDR = True
-except ImportError:
-    HAS_NETADDR = False
-
 
 class Parameters(AnsibleF5Parameters):
     api_map = {
@@ -358,24 +370,26 @@ class Parameters(AnsibleF5Parameters):
         'slowRampTime': 'slow_ramp_time',
         'reselectTries': 'reselect_tries',
         'serviceDownAction': 'service_down_action',
-        'monitor': 'monitors'
+        'monitor': 'monitors',
+        'minActiveMembers': 'priority_group_activation'
     }
 
     api_attributes = [
         'description', 'name', 'loadBalancingMode', 'monitor', 'slowRampTime',
-        'reselectTries', 'serviceDownAction', 'metadata'
+        'reselectTries', 'serviceDownAction', 'metadata', 'minActiveMembers'
     ]
 
     returnables = [
         'monitor_type', 'quorum', 'monitors', 'service_down_action',
         'description', 'lb_method', 'slow_ramp_time',
-        'reselect_tries', 'monitor', 'name', 'partition', 'metadata'
+        'reselect_tries', 'monitor', 'name', 'partition', 'metadata',
+        'priority_group_activation'
     ]
 
     updatables = [
         'monitor_type', 'quorum', 'monitors', 'service_down_action',
         'description', 'lb_method', 'slow_ramp_time', 'reselect_tries',
-        'metadata'
+        'metadata', 'priority_group_activation'
     ]
 
     @property
@@ -389,23 +403,6 @@ class Parameters(AnsibleF5Parameters):
             raise F5ModuleError('Provided lb_method is unknown')
         return lb_method
 
-    def _fqdn_name(self, value):
-        if value is not None and not value.startswith('/'):
-            return '/{0}/{1}'.format(self.partition, value)
-        return value
-
-    @property
-    def monitors(self):
-        if self._values['monitors'] is None:
-            return None
-        monitors = [self._fqdn_name(x) for x in self.monitors_list]
-        if self.monitor_type == 'm_of_n':
-            monitors = ' '.join(monitors)
-            result = 'min %s of { %s }' % (self.quorum, monitors)
-        else:
-            result = ' and '.join(monitors).strip()
-        return result
-
     def _verify_quorum_type(self, quorum):
         try:
             if quorum is None:
@@ -416,6 +413,24 @@ class Parameters(AnsibleF5Parameters):
                 "The specified 'quorum' must be an integer."
             )
 
+    @property
+    def monitors(self):
+        if self._values['monitors'] is None:
+            return None
+        monitors = [fq_name(self.partition, x) for x in self.monitors_list]
+        if self.monitor_type == 'm_of_n':
+            monitors = ' '.join(monitors)
+            result = 'min %s of { %s }' % (self.quorum, monitors)
+        else:
+            result = ' and '.join(monitors).strip()
+        return result
+
+    @property
+    def priority_group_activation(self):
+        if self._values['priority_group_activation'] is None:
+            return None
+        return int(self._values['priority_group_activation'])
+
 
 class ApiParameters(Parameters):
     @property
@@ -447,7 +462,7 @@ class ApiParameters(Parameters):
         if self._values['monitors'] is None:
             return []
         try:
-            result = re.findall(r'/\w+/[^\s}]+', self._values['monitors'])
+            result = re.findall(r'/[\w-]+/[^\s}]+', self._values['monitors'])
             return result
         except Exception:
             return self._values['monitors']
@@ -534,7 +549,7 @@ class UsableChanges(Changes):
 class ReportableChanges(Changes):
     @property
     def monitors(self):
-        result = sorted(re.findall(r'/\w+/[^\s}]+', self._values['monitors']))
+        result = sorted(re.findall(r'/[\w-]+/[^\s}]+', self._values['monitors']))
         return result
 
     @property
@@ -794,6 +809,8 @@ class ModuleManager(object):
             raise F5ModuleError(
                 "When using a 'monitor_type' of 'single', only one monitor may be provided"
             )
+        if self.want.priority_group_activation is None:
+            self.want.update({'priority_group_activation': 0})
 
         self._set_changed_options()
         if self.module.check_mode:
@@ -903,6 +920,10 @@ class ArgumentSpec(object):
             partition=dict(
                 default='Common',
                 fallback=(env_fallback, ['F5_PARTITION'])
+            ),
+            priority_group_activation=dict(
+                type='int',
+                aliases=['minimum_active_members']
             )
         )
         self.argument_spec = {}
@@ -919,8 +940,6 @@ def main():
     )
     if not HAS_F5SDK:
         module.fail_json(msg="The python f5-sdk module is required")
-    if not HAS_NETADDR:
-        module.fail_json(msg="The python netaddr module is required")
 
     try:
         client = F5Client(**module.params)
