@@ -64,7 +64,7 @@ options:
     - Whether to force a reboot (even when the system is already booted).
     - As a safeguard, without force, hpilo_boot will refuse to reboot a server that is already running.
     default: no
-    choices: [ "yes", "no" ]
+    type: bool
   ssl_version:
     description:
       - Change the ssl_version used.
