@@ -311,7 +311,7 @@ def main():
             # FIXME: ERRORS
 
             if rc != 0:
-                module.fail_json(msg="Failed to %s service: %s" % (action, name))
+                module.fail_json(msg="Failed to %s service: %s" % (action, name), rc=rc, stdout=out, stderr=err)
 
             return (rc, out, err)
 
